unit GX_ConvFav;

interface

procedure ConvertFavFiles(const DataPath: string);

implementation

uses
  SysUtils, Classes, Dialogs, FileCtrl,
  OmniXML, GX_ConvFavUtil;

const // Do not localize any of the strings in this const section:
  {$IFNDEF VER140}
  sLineBreak = #13#10;
  {$ENDIF VER140}

  OldFaveFavFile = 'Fave.fav'; // Do not localize.
  NewFaveFavFile = 'FavoriteFiles.xml'; // Do not localize.

  cXmlAttribute_FolderName = 'Name';
  cXmlAttribute_FolderType = 'FolderType';
  cXmlAttribute_FileName = 'Name';
  cXmlAttribute_FileFileName = 'FileName';
  cXmlAttribute_FileDescription = 'Description';
  cXmlAttribute_FileExecType = 'ExecType';
  cXmlAttribute_FileExecProg = 'ExecProg';

  cXmlNode_Root = 'FavoriteFiles';
  cXmlNode_Folder = 'Folder';
  cXmlNode_File = 'File';

  cXmlExecTypeNames: array[TExecType] of string = ('LoadInIDE', 'ShellExecute', 'Custom', 'Project');
  cXmlFolderTypeNames: array[TFolderType] of string = ('Normal', 'Source', 'Bitmaps', 'Glyphs', 'Documentation');

function LoadEntries(const OldFile: string): Boolean;
var
  Stream: TStream;
  ErrorMsg: string;
begin
  Result := False;
  Stream := TFileStream.Create(OldFile, fmOpenRead or fmShareDenyWrite);
  try
    try
      Stream.ReadComponent(Root);
    except on E: Exception do
      begin
        ErrorMsg := 'Unable to load the file:' + sLineBreak + OldFile + sLineBreak +
          'Error: ' + E.Message + sLineBreak + 'No XML file created.';
        MessageDlg(ErrorMsg, mtError, [mbYes, mbNo], 0);
        Exit;
      end;
    end;
  finally
    Stream.Free;
  end;
  Result := True;
end;

procedure SaveEntries(const NewFile: string);

  procedure SetFolderAttributes(const Files: TGXFiles; const FolderNode: IXMLElement);
  begin
    FolderNode.SetAttribute(cXmlAttribute_FolderName, Files.Folder);
    FolderNode.SetAttribute(cXmlAttribute_FolderType, cXmlFolderTypeNames[Files.FolderType]);
  end;

  procedure SetFileAttributes(const AFile: TGXFile; const FileNode: IXMLElement);
  begin
    FileNode.SetAttribute(cXmlAttribute_FileName, AFile.DName);
    FileNode.SetAttribute(cXmlAttribute_FileFileName, AFile.FileName);
    // TODO: Description as text node instead of attribute to make
    // multiline descriptions possible?
    if AFile.Description <> '' then
      FileNode.SetAttribute(cXmlAttribute_FileDescription, AFile.Description);
    FileNode.SetAttribute(cXmlAttribute_FileExecType, cXmlExecTypeNames[AFile.ExecType]);
    if AFile.ExecType = etCustom then
      FileNode.SetAttribute(cXmlAttribute_FileExecProg, AFile.ExecProg);
  end;

  procedure SaveFolder(const Folder: TGXFiles; const Doc: IXMLDocument;
    const ParentNode: IXMLElement);
  var
    Comp: TComponent;
    FolderNode, FileNode: IXMLElement;
    i: Integer;
  begin
    FolderNode := Doc.CreateElement(cXmlNode_Folder);
    SetFolderAttributes(TGXFiles(Folder), FolderNode);
    ParentNode.AppendChild(FolderNode);

    for i := 0 to Folder.ComponentCount - 1 do
    begin
      Comp := Folder.Components[i];
      if Comp is TGXFiles then
      begin
        SaveFolder(TGXFiles(Comp), Doc, FolderNode);
      end
      else
      begin
        Assert(Comp is TGXFile);
        FileNode := Doc.CreateElement(cXmlNode_File);
        SetFileAttributes(TGXFile(Comp), FileNode);
        FolderNode.AppendChild(FileNode);
      end;
    end;
  end;

resourcestring
  SSaveError = 'Your favorite file settings could not be saved:' + sLineBreak +
    sLineBreak +
    '  %s' + sLineBreak +
    sLineBreak +
    'Please verify that the path exists and that the file can ' +
    'be written to; optionally choose a different file name by ' +
    'selecting Options | Options...';
var
  Doc: IXMLDocument;
  RootNode: IXMLElement;
  Header: IXMLProcessingInstruction;
begin
  try
    Doc := CreateXMLDoc;
    Header := Doc.CreateProcessingInstruction('xml', Format('version="1.0" encoding="%s"', ['UTF-8']));
    Doc.AppendChild(Header);

    RootNode := Doc.CreateElement(cXmlNode_Root);
    Doc.DocumentElement := RootNode;
    SaveFolder(Root, Doc, RootNode);

    Doc.Save(NewFile, ofIndent);
  except
    on E: Exception do
    begin
      MessageDlg(Format(SSaveError, [E.Message]), mtError, [mbOK], 0);
    end;
  end;
end;

procedure ConvertFavFiles(const DataPath: string);
resourcestring
  SFavFileNotFound = 'Couldn''t find the file %s in the specified folder %s.' + sLineBreak +
    'Please check if the folder is correct.';
begin
  if not FileExists(DataPath + OldFaveFavFile) then
  begin
    MessageDlg(Format(SFavFileNotFound, [OldFaveFavFile, DataPath]), mtWarning, [mbOK], 0);
    Exit;
  end;

  Root := TGXFiles.Create(nil);
  if LoadEntries(DataPath + OldFaveFavFile) then
    SaveEntries(DataPath + NewFaveFavFile);
  FreeAndNil(Root);
end;

end.

